@@ -7,6 +7,7 @@ from conans.util.files import rmdir, mkdir
 from conans.client.remote_registry import RemoteRegistry
 from conans import tools
 from conans.errors import ConanException
+import subprocess
 
 
 def _hide_password(resource):
@@ -38,11 +39,17 @@ def _handle_profiles(source_folder, target_folder, output):
             shutil.copy(os.path.join(root, f), os.path.join(target_folder, profile))
 
 
-def _process_git_repo(repo_url, client_cache, output, runner, tmp_folder, verify_ssl):
+def _process_git_repo(repo_url, client_cache, output, tmp_folder, verify_ssl):
     output.info("Trying to clone repo  %s" % repo_url)
 
     with tools.chdir(tmp_folder):
-        runner('git -c http.sslVerify=%s clone "%s" config' % (verify_ssl, repo_url), output=output)
+        try:
+            subprocess.check_output('git -c http.sslVerify=%s clone "%s" config' % (verify_ssl, repo_url),
+                                    shell=True)
+            output.info("Repo cloned")
+        except Exception as e:
+            raise ConanException("config install error. Can't clone repo: %s" % str(e))
+
     tmp_folder = os.path.join(tmp_folder, "config")
     _process_folder(tmp_folder, client_cache, output)
 
@@ -97,7 +104,7 @@ def _process_download(item, client_cache, output, tmp_folder, verify_ssl):
     _process_zip_file(zippath, client_cache, output, tmp_folder, remove=True)
 
 
-def configuration_install(item, client_cache, output, runner, verify_ssl):
+def configuration_install(item, client_cache, output, verify_ssl):
     tmp_folder = os.path.join(client_cache.conan_folder, "tmp_config_install")
     # necessary for Mac OSX, where the temp folders in /var/ are symlinks to /private/var/
     tmp_folder = os.path.realpath(tmp_folder)
@@ -111,7 +118,7 @@ def configuration_install(item, client_cache, output, runner, verify_ssl):
                                      "'general.config_install' not defined in conan.conf")
 
         if item.endswith(".git"):
-            _process_git_repo(item, client_cache, output, runner, tmp_folder, verify_ssl)
+            _process_git_repo(item, client_cache, output, tmp_folder, verify_ssl)
         elif os.path.exists(item):
             # is a local file
             _process_zip_file(item, client_cache, output, tmp_folder)
