@@ -3,13 +3,13 @@ import os
 from collections import Counter
 
 from conans.client import packager
-from conans.client.build_requires import BuildRequires
+from conans.client.graph.build_requires import BuildRequires
 from conans.client.client_cache import ClientCache
-from conans.client.cmd.export import cmd_export
-from conans.client.deps_builder import DepsGraphBuilder
+from conans.client.cmd.export import _execute_export
+from conans.client.graph.graph_builder import DepsGraphBuilder
 from conans.client.generators import write_generators
 from conans.client.generators.text import TXTGenerator
-from conans.client.importer import run_imports, undo_imports, run_deploy
+from conans.client.importer import run_imports, run_deploy
 from conans.client.installer import ConanInstaller, call_system_requirements
 from conans.client.loader import ConanFileLoader
 from conans.client.manifest_manager import ManifestManager
@@ -17,22 +17,27 @@ from conans.client.output import ScopedOutput, Color
 from conans.client.printer import Printer
 from conans.client.profile_loader import read_conaninfo_profile
 from conans.client.proxy import ConanProxy
-from conans.client.remote_registry import RemoteRegistry
 from conans.client.remover import ConanRemover
-from conans.client.require_resolver import RequireResolver
-from conans.client.source import config_source_local
+from conans.client.graph.require_resolver import RequireResolver
+from conans.client.source import config_source_local, complete_recipe_sources
 from conans.client.tools import cross_building, get_cross_building_settings
 from conans.client.userio import UserIO
 from conans.errors import NotFoundException, ConanException, conanfile_exception_formatter
 from conans.model.conan_file import get_env_context_manager
-from conans.model.manifest import FileTreeManifest
 from conans.model.ref import ConanFileReference, PackageReference
-from conans.paths import CONANFILE, CONANINFO, CONANFILE_TXT, CONAN_MANIFEST, BUILD_INFO
+from conans.paths import CONANFILE, CONANINFO, CONANFILE_TXT, BUILD_INFO
 from conans.util.files import save, rmdir, normalize, mkdir, load
 from conans.util.log import logger
+from conans.client.loader_parse import load_conanfile_class
 
 
 class BuildMode(object):
+    """ build_mode => ["*"] if user wrote "--build"
+                   => ["hello*", "bye*"] if user wrote "--build hello --build bye"
+                   => False if user wrote "never"
+                   => True if user wrote "missing"
+                   => "outdated" if user wrote "--build outdated"
+    """
     def __init__(self, params, output):
         self._out = output
         self.outdated = False
@@ -98,7 +103,7 @@ class ConanManager(object):
     business logic
     """
     def __init__(self, client_cache, user_io, runner, remote_manager, search_manager,
-                 settings_preprocessor):
+                 settings_preprocessor, recorder, registry):
         assert isinstance(user_io, UserIO)
         assert isinstance(client_cache, ClientCache)
         self._client_cache = client_cache
@@ -107,6 +112,8 @@ class ConanManager(object):
         self._remote_manager = remote_manager
         self._search_manager = search_manager
         self._settings_preprocessor = settings_preprocessor
+        self._recorder = recorder
+        self._registry = registry
 
     def _load_consumer_conanfile(self, conanfile_path, info_folder, output,
                                  deps_info_required=False):
@@ -150,32 +157,32 @@ class ConanManager(object):
             self._settings_preprocessor.preprocess(cache_settings)
         return ConanFileLoader(self._runner, cache_settings, profile)
 
-    def export(self, conanfile_path, name, version, user, channel,  keep_source=False):
-        cmd_export(conanfile_path, name, version, user, channel, keep_source,
-                   self._user_io.out, self._client_cache)
+    def get_proxy(self, remote_name=None, manifest_manager=None):
+        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager,
+                                  remote_name=remote_name, recorder=self._recorder, registry=self._registry,
+                                  manifest_manager=manifest_manager)
+        return remote_proxy
 
-    def export_pkg(self, reference, source_folder, build_folder, install_folder, profile, force):
+    def export_pkg(self, reference, source_folder, build_folder, package_folder, install_folder, profile, force):
 
         conan_file_path = self._client_cache.conanfile(reference)
         if not os.path.exists(conan_file_path):
             raise ConanException("Package recipe '%s' does not exist" % str(reference))
 
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager,
-                                  remote_name=None, update=False, check_updates=False,
-                                  manifest_manager=None)
+        remote_proxy = self.get_proxy()
 
         loader = self.get_loader(profile)
         conanfile = loader.load_virtual([reference], scope_options=True)
         if install_folder and existing_info_files(install_folder):
             _load_deps_info(install_folder, conanfile, required=True)
 
-        graph_builder = self._get_graph_builder(loader, False, remote_proxy)
-        deps_graph = graph_builder.load(conanfile)
+        graph_builder = self._get_graph_builder(loader, remote_proxy)
+        deps_graph = graph_builder.load_graph(conanfile, check_updates=False, update=False)
 
         # this is a bit tricky, but works. The loading of a cache package makes the referenced
         # one, the first of the first level, always existing
         nodes = deps_graph.direct_requires()
-        _, conanfile = nodes[0]
+        conanfile = nodes[0].conanfile
         pkg_id = conanfile.info.package_id()
         self._user_io.out.info("Packaging to %s" % pkg_id)
         pkg_reference = PackageReference(reference, pkg_id)
@@ -192,27 +199,38 @@ class ConanManager(object):
         recipe_hash = self._client_cache.load_manifest(reference).summary_hash
         conanfile.info.recipe_hash = recipe_hash
         conanfile.develop = True
-        if source_folder or build_folder:
-            install_folder = build_folder  # conaninfo.txt will be there
-            package_output = ScopedOutput(str(reference), self._user_io.out)
+        package_output = ScopedOutput(str(reference), self._user_io.out)
+        if package_folder:
+            packager.export_pkg(conanfile, package_folder, dest_package_folder, package_output)
+        else:
             packager.create_package(conanfile, source_folder, build_folder, dest_package_folder,
                                     install_folder, package_output, local=True)
 
-    def download(self, reference, package_ids, remote):
+    def download(self, reference, package_ids, remote_name, recipe):
         """ Download conanfile and specified packages to local repository
         @param reference: ConanFileReference
         @param package_ids: Package ids or empty for download all
         @param remote: install only from that remote
+        @param only_recipe: download only the recipe
         """
         assert(isinstance(reference, ConanFileReference))
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager, remote)
+        remote_proxy = self.get_proxy(remote_name=remote_name)
         remote, _ = remote_proxy._get_remote()
         package = self._remote_manager.search_recipes(remote, reference, None)
         if not package:  # Search the reference first, and raise if it doesn't exist
             raise ConanException("'%s' not found in remote" % str(reference))
 
         # First of all download package recipe
-        remote_proxy.get_recipe(reference)
+        remote_proxy.get_recipe(reference, check_updates=True, update=True)
+
+        if recipe:
+            return
+
+        # Download the sources too, don't be lazy
+        conan_file_path = self._client_cache.conanfile(reference)
+        conanfile = load_conanfile_class(conan_file_path)
+        complete_recipe_sources(self._remote_manager, self._client_cache, self._registry,
+                                conanfile, reference)
 
         if package_ids:
             remote_proxy.download_packages(reference, package_ids)
@@ -236,34 +254,36 @@ class ConanManager(object):
             require.conan_reference = require.range_reference = inject_require
         else:
             conanfile.requires(str(inject_require))
+        conanfile._user = inject_require.user
+        conanfile._channel = inject_require.channel
 
-    def _get_graph_builder(self, loader, update, remote_proxy):
+    def _get_graph_builder(self, loader, remote_proxy):
         local_search = self._search_manager
-        resolver = RequireResolver(self._user_io.out, local_search, remote_proxy, update=update)
+        resolver = RequireResolver(self._user_io.out, local_search, remote_proxy)
         graph_builder = DepsGraphBuilder(remote_proxy, self._user_io.out, loader, resolver)
         return graph_builder
 
-    def _get_deps_graph(self, reference, profile, remote_proxy):
+    def _get_deps_graph(self, reference, profile, remote_proxy, check_updates, update):
         loader = self.get_loader(profile)
         conanfile = self._load_install_conanfile(loader, reference)
-        graph_builder = self._get_graph_builder(loader, False, remote_proxy)
-        deps_graph = graph_builder.load(conanfile)
+        graph_builder = self._get_graph_builder(loader, remote_proxy)
+        deps_graph = graph_builder.load_graph(conanfile, check_updates, update)
         return deps_graph, graph_builder, conanfile
 
-    def info_build_order(self, reference, profile, build_order, remote, check_updates):
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager, remote,
-                                  update=False, check_updates=check_updates)
-        deps_graph, _, _ = self._get_deps_graph(reference, profile, remote_proxy)
+    def info_build_order(self, reference, profile, build_order, remote_name, check_updates):
+        remote_proxy = self.get_proxy(remote_name=remote_name)
+        deps_graph, _, _ = self._get_deps_graph(reference, profile, remote_proxy, update=False,
+                                                check_updates=check_updates)
         result = deps_graph.build_order(build_order)
         return result
 
-    def info_nodes_to_build(self, reference, profile, build_modes, remote, check_updates):
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager, remote,
-                                  update=False, check_updates=check_updates)
-        deps_graph, _, conanfile = self._get_deps_graph(reference, profile, remote_proxy)
+    def info_nodes_to_build(self, reference, profile, build_modes, remote_name, check_updates):
+        remote_proxy = self.get_proxy(remote_name=remote_name)
+        deps_graph, _, conanfile = self._get_deps_graph(reference, profile, remote_proxy, update=False,
+                                                        check_updates=check_updates)
         build_mode = BuildMode(build_modes, self._user_io.out)
         installer = ConanInstaller(self._client_cache, self._user_io.out, remote_proxy, build_mode,
-                                   None)
+                                   None, recorder=self._recorder)
         nodes = installer.nodes_to_build(deps_graph)
         counter = Counter(ref.conan.name for ref, _ in nodes)
         ret = [ref if counter[ref.conan.name] > 1 else str(ref.conan) for ref, _ in nodes]
@@ -277,19 +297,14 @@ class ConanManager(object):
 
         return project_reference
 
-    def info_get_graph(self, reference, profile, remote=None, check_updates=False):
+    def info_get_graph(self, reference, profile, remote_name=None, check_updates=False):
         """ Fetch and build all dependencies for the given reference
         @param reference: ConanFileReference or path to user space conanfile
-        @param current_path: where the output files will be saved
         @param remote: install only from that remote
-        @param profile: Profile object with both the -s introduced options and profile readed values
-        @param build_modes: List of build_modes specified
         """
-
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager, remote,
-                                  update=False, check_updates=check_updates)
-
-        deps_graph, graph_builder, conanfile = self._get_deps_graph(reference, profile, remote_proxy)
+        remote_proxy = self.get_proxy(remote_name=remote_name)
+        deps_graph, graph_builder, conanfile = self._get_deps_graph(reference, profile, remote_proxy,
+                                                                    update=False, check_updates=check_updates)
 
         if check_updates:
             graph_updates_info = graph_builder.get_graph_updates_info(deps_graph)
@@ -298,7 +313,7 @@ class ConanManager(object):
 
         return deps_graph, graph_updates_info, self._get_project_reference(reference, conanfile)
 
-    def install(self, reference, install_folder, profile, remote=None, build_modes=None,
+    def install(self, reference, install_folder, profile, remote_name=None, build_modes=None,
                 update=False, manifest_folder=None, manifest_verify=False,
                 manifest_interactive=False, generators=None, no_imports=False, inject_require=None,
                 install_reference=False, keep_build=False):
@@ -318,6 +333,7 @@ class ConanManager(object):
         @param no_imports: Install specified packages but avoid running imports
         @param inject_require: Reference to add as a requirement to the conanfile
         """
+
         if generators is not False:
             generators = set(generators) if generators else set()
             generators.add("txt")  # Add txt generator by default
@@ -326,8 +342,7 @@ class ConanManager(object):
                                            client_cache=self._client_cache,
                                            verify=manifest_verify,
                                            interactive=manifest_interactive) if manifest_folder else None
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager, remote,
-                                  update=update, manifest_manager=manifest_manager)
+        remote_proxy = self.get_proxy(remote_name=remote_name, manifest_manager=manifest_manager)
 
         loader = self.get_loader(profile)
         if not install_reference:
@@ -338,10 +353,8 @@ class ConanManager(object):
         conanfile = self._load_install_conanfile(loader, reference)
         if inject_require:
             self._inject_require(conanfile, inject_require)
-        graph_builder = self._get_graph_builder(loader, update, remote_proxy)
-        deps_graph = graph_builder.load(conanfile)
-
-        registry = RemoteRegistry(self._client_cache.registry, self._user_io.out)
+        graph_builder = self._get_graph_builder(loader, remote_proxy)
+        deps_graph = graph_builder.load_graph(conanfile, False, update)
 
         if not isinstance(reference, ConanFileReference):
             output = ScopedOutput(("%s (test package)" % str(inject_require)) if inject_require else "PROJECT",
@@ -350,7 +363,7 @@ class ConanManager(object):
         else:
             output = ScopedOutput(str(reference), self._user_io.out)
             output.highlight("Installing package")
-        Printer(self._user_io.out).print_graph(deps_graph, registry)
+        Printer(self._user_io.out).print_graph(deps_graph, self._registry)
 
         try:
             if cross_building(loader._settings):
@@ -361,15 +374,15 @@ class ConanManager(object):
             pass
 
         build_mode = BuildMode(build_modes, self._user_io.out)
-        build_requires = BuildRequires(loader, graph_builder, registry)
+        build_requires = BuildRequires(loader, graph_builder, self._registry)
         installer = ConanInstaller(self._client_cache, output, remote_proxy, build_mode,
-                                   build_requires)
+                                   build_requires, recorder=self._recorder)
 
         # Apply build_requires to consumer conanfile
         if not isinstance(reference, ConanFileReference):
-            build_requires.install("", conanfile, installer, profile.build_requires, output)
+            build_requires.install("", conanfile, installer, profile.build_requires, output, update)
 
-        installer.install(deps_graph, profile.build_requires, keep_build)
+        installer.install(deps_graph, profile.build_requires, keep_build, update=update)
         build_mode.report_matches()
 
         if install_folder:
@@ -408,11 +421,13 @@ class ConanManager(object):
         output = ScopedOutput("PROJECT", self._user_io.out)
         # only infos if exist
         conanfile = self._load_consumer_conanfile(conanfile_path, info_folder, output)
+        conanfile_folder = os.path.dirname(conanfile_path)
+        if conanfile_folder != source_folder:
+            output.info("Executing exports to: %s" % source_folder)
+            _execute_export(conanfile_path, conanfile, source_folder,
+                            source_folder, output)
         config_source_local(source_folder, conanfile, output)
 
-    def imports_undo(self, current_path):
-        undo_imports(current_path, self._user_io.out)
-
     def imports(self, conan_file_path, dest_folder, info_folder):
         """
         :param conan_file_path: Abs path to a conanfile
@@ -439,7 +454,7 @@ class ConanManager(object):
                                 install_folder, output, local=True, copy_info=True)
 
     def build(self, conanfile_path, source_folder, build_folder, package_folder, install_folder,
-              test=False):
+              test=False, should_configure=True, should_build=True, should_install=True):
         """ Call to build() method saved on the conanfile.py
         param conanfile_path: path to a conanfile.py
         """
@@ -465,6 +480,10 @@ class ConanManager(object):
             except ConanException:
                 pass
 
+        conan_file.should_configure = should_configure
+        conan_file.should_build = should_build
+        conan_file.should_install = should_install
+
         try:
             mkdir(build_folder)
             os.chdir(build_folder)
@@ -488,7 +507,7 @@ class ConanManager(object):
             raise ConanException("Unable to build it successfully\n%s" % '\n'.join(trace[3:]))
 
     def remove(self, pattern, src=False, build_ids=None, package_ids_filter=None, force=False,
-               remote=None, packages_query=None, outdated=False):
+               remote_name=None, packages_query=None, outdated=False):
         """ Remove conans and/or packages
         @param pattern: string to match packages
         @param src: Remove src folder
@@ -498,43 +517,12 @@ class ConanManager(object):
         @param force: if True, it will be deleted without requesting anything
         @param packages_query: Only if src is a reference. Query settings and options
         """
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager, remote)
-        remover = ConanRemover(self._client_cache, self._remote_manager, self._user_io, remote_proxy)
-        remover.remove(pattern, remote, src, build_ids, package_ids_filter, force=force,
+        remote_proxy = self.get_proxy(remote_name=remote_name)
+        remover = ConanRemover(self._client_cache, self._remote_manager, self._user_io, remote_proxy,
+                               self._registry)
+        remover.remove(pattern, remote_name, src, build_ids, package_ids_filter, force=force,
                        packages_query=packages_query, outdated=outdated)
 
-    def user(self, remote=None, name=None, password=None):
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager, remote)
-        if password == "":
-            if not remote:
-                remote = remote_proxy.registry.default_remote.name
-            name, password = self._user_io.request_login(remote_name=remote, username=name)
-        return remote_proxy.authenticate(name, password)
-
-    def get_path(self, reference, package_id=None, path=None, remote=None):
-        remote_proxy = ConanProxy(self._client_cache, self._user_io, self._remote_manager, remote)
-        if not path and not package_id:
-            path = "conanfile.py"
-        elif not path and package_id:
-            path = "conaninfo.txt"
-        return remote_proxy.get_path(reference, package_id, path), path
-
-    def export_alias(self, reference, target_reference):
-
-        conanfile = """
-from conans import ConanFile
-
-class AliasConanfile(ConanFile):
-    alias = "%s"
-""" % str(target_reference)
-
-        export_path = self._client_cache.export(reference)
-        mkdir(export_path)
-        save(os.path.join(export_path, CONANFILE), conanfile)
-        mkdir(self._client_cache.export_sources(reference))
-        digest = FileTreeManifest.create(export_path)
-        save(os.path.join(export_path, CONAN_MANIFEST), str(digest))
-
 
 def _load_deps_info(current_path, conanfile, required):
 
