@@ -3,23 +3,30 @@ import json
 import os
 import platform
 import re
+import deprecation
 
 import subprocess
 from contextlib import contextmanager
 
 from conans.client.tools.env import environment_append
-from conans.client.tools.oss import cpu_count, detected_architecture, os_info
+from conans.client.tools.oss import detected_architecture, os_info
 from conans.errors import ConanException
 from conans.util.env_reader import get_env
-from conans.util.files import decode_text, load
+from conans.util.files import decode_text, save, mkdir_tmp
+from conans.unicode import get_cwd
 
 _global_output = None
 
 
+@deprecation.deprecated(deprecated_in="1.2", removed_in="2.0",
+                        details="Use the MSBuild() build helper instead")
 def msvc_build_command(settings, sln_path, targets=None, upgrade_project=True, build_type=None,
                        arch=None, parallel=True, force_vcvars=False, toolset=None, platforms=None):
     """ Do both: set the environment variables and call the .sln build
     """
+    import warnings
+    warnings.warn("deprecated", DeprecationWarning)
+
     vcvars = vcvars_command(settings, force=force_vcvars)
     build = build_sln_command(settings, sln_path, targets, upgrade_project, build_type, arch,
                               parallel, toolset=toolset, platforms=platforms)
@@ -27,6 +34,8 @@ def msvc_build_command(settings, sln_path, targets=None, upgrade_project=True, b
     return command
 
 
+@deprecation.deprecated(deprecated_in="1.2", removed_in="2.0",
+                        details="Use the MSBuild() build helper instead")
 def build_sln_command(settings, sln_path, targets=None, upgrade_project=True, build_type=None,
                       arch=None, parallel=True, toolset=None, platforms=None):
     """
@@ -35,53 +44,20 @@ def build_sln_command(settings, sln_path, targets=None, upgrade_project=True, bu
         command = "%s && %s" % (tools.vcvars_command(self.settings), build_command)
         self.run(command)
     """
-    targets = targets or []
-    command = ""
+    from conans.client.build.msbuild import MSBuild
+    tmp = MSBuild(settings)
+    tmp._output = _global_output
 
-    if upgrade_project and not get_env("CONAN_SKIP_VS_PROJECTS_UPGRADE", False):
-        command += "devenv %s /upgrade && " % sln_path
-    else:
-        _global_output.info("Skipped sln project upgrade")
+    # Generate the properties file
+    props_file_contents = tmp._get_props_file_contents()
+    tmp_path = os.path.join(mkdir_tmp(), ".conan_properties")
+    save(tmp_path, props_file_contents)
 
-    build_type = build_type or settings.build_type
-    arch = arch or settings.arch
-    if not build_type:
-        raise ConanException("Cannot build_sln_command, build_type not defined")
-    if not arch:
-        raise ConanException("Cannot build_sln_command, arch not defined")
-    command += "msbuild %s /p:Configuration=%s" % (sln_path, build_type)
-    msvc_arch = {'x86': 'x86',
-                 'x86_64': 'x64',
-                 'armv7': 'ARM',
-                 'armv8': 'ARM64'}
-    if platforms:
-        msvc_arch.update(platforms)
-    msvc_arch = msvc_arch.get(str(arch))
-    try:
-        sln = load(sln_path)
-        pattern = re.compile(r"GlobalSection\(SolutionConfigurationPlatforms\)(.*?)EndGlobalSection", re.DOTALL)
-        solution_global = pattern.search(sln).group(1)
-        lines = solution_global.splitlines()
-        lines = [s.split("=")[0].strip() for s in lines]
-    except Exception:
-        pass
-    else:
-        config = "%s|%s" % (build_type, msvc_arch)
-        if config not in "".join(lines):
-            _global_output.warn("***** The configuration %s does not exist in this solution *****" % config)
-            _global_output.warn("Use 'platforms' argument to define your architectures")
-
-    if msvc_arch:
-        command += ' /p:Platform="%s"' % msvc_arch
-
-    if parallel:
-        command += ' /m:%s' % cpu_count()
-
-    if targets:
-        command += " /target:%s" % ";".join(targets)
-
-    if toolset:
-        command += " /p:PlatformToolset=%s" % toolset
+    # Build command
+    command = tmp.get_command(sln_path, tmp_path,
+                              targets=targets, upgrade_project=upgrade_project,
+                              build_type=build_type, arch=arch, parallel=parallel,
+                              toolset=toolset, platforms=platforms, use_env=False)
 
     return command
 
@@ -291,7 +267,7 @@ def vcvars_command(settings, arch=None, compiler_version=None, force=False):
         vs_path = vs_installation_path(str(compiler_version))
 
         if not vs_path or not os.path.isdir(vs_path):
-            _global_output.warn("VS non-existing installation")
+            raise ConanException("VS non-existing installation: Visual Studio %s" % str(compiler_version))
         else:
             vcvars_path = ""
             if int(compiler_version) > 14:
@@ -346,8 +322,11 @@ def vcvars_dict(settings, arch=None, compiler_version=None, force=False, filter_
 
 @contextmanager
 def vcvars(*args, **kwargs):
-    new_env = vcvars_dict(*args, **kwargs)
-    with environment_append(new_env):
+    if platform.system() == "Windows":
+        new_env = vcvars_dict(*args, **kwargs)
+        with environment_append(new_env):
+            yield
+    else:
         yield
 
 
@@ -456,12 +435,13 @@ def run_in_windows_bash(conanfile, bashcmd, cwd=None, subsystem=None, msys_mingw
 
         # Needed to change to that dir inside the bash shell
         if cwd and not os.path.isabs(cwd):
-            cwd = os.path.join(os.getcwd(), cwd)
+            cwd = os.path.join(get_cwd(), cwd)
 
-        curdir = unix_path(cwd or os.getcwd(), path_flavor=subsystem)
+        curdir = unix_path(cwd or get_cwd(), path_flavor=subsystem)
         to_run = 'cd "%s"%s && %s ' % (curdir, hack_env, bashcmd)
         bash_path = os_info.bash_path()
         bash_path = '"%s"' % bash_path if " " in bash_path else bash_path
         wincmd = '%s --login -c %s' % (bash_path, escape_windows_cmd(to_run))
         conanfile.output.info('run_in_windows_bash: %s' % wincmd)
-        return conanfile.run(wincmd, win_bash=False)
+        # https://github.com/conan-io/conan/issues/2839 (subprocess=True)
+        return conanfile._runner(wincmd, output=conanfile.output, subprocess=True)
