@@ -1,3 +1,4 @@
+import json
 import unittest
 from conans.test.utils.tools import TestClient, TestServer, TestRequester
 from conans.test.utils.test_files import hello_source_files, temp_folder,\
@@ -7,7 +8,7 @@ import os
 from conans.paths import CONAN_MANIFEST, EXPORT_TGZ_NAME, CONANINFO
 import platform
 import stat
-from conans.util.files import save, mkdir
+from conans.util.files import load, mkdir, save
 from conans.model.ref import ConanFileReference, PackageReference
 from conans.model.manifest import FileTreeManifest
 from conans.test.utils.test_files import uncompress_packaged_files
@@ -78,11 +79,11 @@ class UploadTest(unittest.TestCase):
         files = hello_source_files()
         self.client.save(files, path=reg_folder)
         self.client.save({CONANFILE: myconan1,
-                          CONAN_MANIFEST: str(conan_digest),
                           "include/math/lib1.h": "//copy",
                           "my_lib/debug/libd.a": "//copy",
                           "my_data/readme.txt": "//copy",
                           "my_bin/executable": "//copy"}, path=reg_folder)
+        conan_digest.save(reg_folder)
         mkdir(self.client.client_cache.export_sources(self.conan_ref))
 
         self.package_ref = PackageReference(self.conan_ref, "myfakeid")
@@ -103,7 +104,7 @@ class UploadTest(unittest.TestCase):
 
         digest_path = self.client.client_cache.digestfile_package(self.package_ref)
         expected_manifest = FileTreeManifest.create(os.path.dirname(digest_path))
-        save(os.path.join(package_folder, CONAN_MANIFEST), str(expected_manifest))
+        expected_manifest.save(package_folder)
 
         self.server_reg_folder = self.test_server.paths.export(self.conan_ref)
         self.assertFalse(os.path.exists(self.server_reg_folder))
@@ -211,7 +212,7 @@ class UploadTest(unittest.TestCase):
         pack_path = self.client.paths.package(self.package_ref)
         digest_path = self.client.client_cache.digestfile_package(self.package_ref)
         expected_manifest = FileTreeManifest.create(os.path.dirname(digest_path))
-        save(os.path.join(pack_path, CONAN_MANIFEST), str(expected_manifest))
+        expected_manifest.save(pack_path)
 
         self.client.run("upload %s --all" % str(self.conan_ref), ignore_error=False)
         self.assertIn("Compressing recipe", self.client.user_io.out)
@@ -239,16 +240,16 @@ class TestConan(ConanFile):
         self.client.run("upload Hello/1.2@lasote/stable", ignore_error=False)
         self.assertIn("Uploading conanmanifest.txt", self.client.user_io.out)
 
+    def single_binary_test(self):
+        """ basic installation of a new conans
+        """
+        # Try to upload an package without upload conans first
+        self.client.run('upload %s -p %s' % (self.conan_ref, str(self.package_ref.package_id)))
+        self.assertIn("Uploaded conan recipe '%s'" % str(self.conan_ref), self.client.out)
+
     def simple_test(self):
         """ basic installation of a new conans
         """
-
-        # Try to upload an package without upload conans first
-        self.client.run('upload %s -p %s' % (self.conan_ref, str(self.package_ref.package_id)),
-                        ignore_error=True)
-        self.assertIn("There are no remote conanfiles like %s" % str(self.conan_ref),
-                      self.client.user_io.out)
-
         # Upload conans
         self.client.run('upload %s' % str(self.conan_ref))
         self.assertTrue(os.path.exists(self.server_reg_folder))
@@ -305,7 +306,7 @@ class TestConan(ConanFile):
         self.client.run('upload %s --all' % str(self.conan_ref))
         lines = [line.strip() for line in str(self.client.user_io.out).splitlines()
                  if line.startswith("Uploading")]
-        self.assertEqual(lines, ["Uploading Hello/1.2.1@frodo/stable",
+        self.assertEqual(lines, ["Uploading Hello/1.2.1@frodo/stable to remote 'default'",
                                  "Uploading conanmanifest.txt",
                                  "Uploading conanfile.py",
                                  "Uploading conan_export.tgz",
@@ -325,11 +326,10 @@ class TestConan(ConanFile):
         self.assertTrue(os.path.exists(self.server_pack_folder))
 
         # Fake datetime from exported date and upload again
-        digest_path = os.path.join(self.client.paths.export(self.conan_ref), CONAN_MANIFEST)
         old_digest = self.client.paths.load_manifest(self.conan_ref)
         old_digest.file_sums["new_file"] = "012345"
         fake_digest = FileTreeManifest(2, old_digest.file_sums)
-        save(digest_path, str(fake_digest))
+        fake_digest.save(self.client.paths.export(self.conan_ref))
 
         self.client.run('upload %s' % str(self.conan_ref), ignore_error=True)
         self.assertIn("Remote recipe is newer than local recipe", self.client.user_io.out)
@@ -337,3 +337,75 @@ class TestConan(ConanFile):
         self.client.run('upload %s --force' % str(self.conan_ref))
         self.assertIn("Uploading %s" % str(self.conan_ref),
                       self.client.user_io.out)
+
+    def upload_json_test(self):
+        conanfile = """
+from conans import ConanFile
+
+class TestConan(ConanFile):
+    name = "test"
+    version = "0.1"
+
+    def package(self):
+        self.copy("mylib.so", dst="lib")
+"""
+
+        client = self._get_client()
+        client.save({"conanfile.py": conanfile,
+                     "mylib.so": ""})
+        client.run("create . danimtb/testing")
+
+        # Test conflict parameter error
+        error = client.run("upload test/0.1@danimtb/* --all -p ewvfw --json upload.json",
+                           ignore_error=True)
+        self.assertTrue(error)
+        json_path = os.path.join(client.current_folder, "upload.json")
+        self.assertTrue(os.path.exists(json_path))
+        json_content = load(json_path)
+        output = json.loads(json_content)
+        self.assertTrue(output["error"])
+        self.assertEqual(0, len(output["uploaded"]))
+
+        # Test invalid reference error
+        error = client.run("upload fake/0.1@danimtb/testing --all --json upload.json",
+                           ignore_error=True)
+        self.assertTrue(error)
+        json_path = os.path.join(client.current_folder, "upload.json")
+        self.assertTrue(os.path.exists(json_path))
+        json_content = load(json_path)
+        output = json.loads(json_content)
+        self.assertTrue(output["error"])
+        self.assertEqual(0, len(output["uploaded"]))
+
+        # Test normal upload
+        client.run("upload test/0.1@danimtb/testing --all --json upload.json")
+        self.assertTrue(os.path.exists(json_path))
+        json_content = load(json_path)
+        output = json.loads(json_content)
+        output_expected = {"error": False,
+                           "uploaded": [
+                               {
+                                   "recipe": {
+                                       "id": "test/0.1@danimtb/testing",
+                                       "remote_url": "unknown",
+                                       "remote_name": "default",
+                                       "time": "unknown"
+                                   },
+                                   "packages": [
+                                       {
+                                           "id": "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9",
+                                           "time": "unknown"
+                                       }
+                                   ]
+                               }
+                           ]}
+        self.assertEqual(output_expected["error"], output["error"])
+        self.assertEqual(len(output_expected["uploaded"]), len(output["uploaded"]))
+
+        for i, item in enumerate(output["uploaded"]):
+            self.assertEqual(output_expected["uploaded"][i]["recipe"]["id"], item["recipe"]["id"])
+            self.assertEqual(output_expected["uploaded"][i]["recipe"]["remote_name"],
+                             item["recipe"]["remote_name"])
+            for j, subitem in enumerate(item["packages"]):
+                self.assertEqual(output_expected["uploaded"][i]["packages"][j]["id"],
+                                 subitem["id"])
